'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _yeomanGenerator = require('yeoman-generator');

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _yosay = require('yosay');

var _yosay2 = _interopRequireDefault(_yosay);

var _mkdirp = require('mkdirp');

var _mkdirp2 = _interopRequireDefault(_mkdirp);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var PhpGenerator = function (_Base) {
  _inherits(PhpGenerator, _Base);

  function PhpGenerator() {
    var _Object$getPrototypeO;

    _classCallCheck(this, PhpGenerator);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    var _this = _possibleConstructorReturn(this, (_Object$getPrototypeO = Object.getPrototypeOf(PhpGenerator)).call.apply(_Object$getPrototypeO, [this].concat(args)));

    _this.argument('appname', { required: false, default: _this.appname });
    return _this;
  }

  _createClass(PhpGenerator, [{
    key: 'prompting',
    value: function prompting() {
      var _this2 = this;

      // Have Yeoman greet the user.
      console.log((0, _yosay2.default)('Welcome to the grand ' + _chalk2.default.red('PHP webapp') + ' generator!'));

      console.log(_chalk2.default.bold.yellow('Let\'s create your awesome \n        ' + _chalk2.default.underline.bold.red(this.appname) + ' project!\n'));

      var prompts = [{
        name: 'author',
        message: 'Author',
        default: ''
      }, {
        name: 'email',
        message: 'Email',
        default: ''
      }, {
        name: 'repo',
        message: 'Repository',
        default: ''
      }, {
        type: 'confirm',
        name: 'doUseCi',
        message: 'Do you want to create PHPCI configuration?',
        default: false
      }, {
        type: 'confirm',
        name: 'doUseDeployer',
        message: 'Do you want to create deployer configuration?',
        default: false
      }];

      var deployerPrompts = [{
        name: 'name',
        message: 'Server alias(only for deployer)',
        default: 'production'
      }, {
        name: 'host',
        message: 'Host',
        default: 0
      }, {
        name: 'port',
        message: 'Server port',
        default: 0
      }, {
        name: 'user',
        message: 'Username',
        default: 'user'
      }, {
        type: 'password',
        name: 'pass',
        message: 'Password',
        default: 'pass'
      }, {
        name: 'stage',
        message: 'stage',
        default: 'production'
      }, {
        name: 'deployPath',
        message: 'Deploy path',
        default: '/var/www/html'
      }, {
        name: 'branch',
        message: 'Branch',
        default: 'master'
      }, {
        type: 'confirm',
        name: 'doSpecifyAnotherServer',
        message: 'Do specify another server?',
        default: false
      }];

      return this.prompt(prompts).then(function (answers) {
        _this2.answers = answers;

        if (answers.doUseDeployer) {
          var _ret = function () {
            _this2.answers.deployer = [];

            var promptLoop = function promptLoop() {
              return _this2.prompt(deployerPrompts).then(function (answers) {
                _this2.answers.deployer.push(answers);

                if (answers.doSpecifyAnotherServer) {
                  return promptLoop();
                }
              });
            };

            return {
              v: promptLoop()
            };
          }();

          if ((typeof _ret === 'undefined' ? 'undefined' : _typeof(_ret)) === "object") return _ret.v;
        }
      });
    }
  }, {
    key: 'writing',
    get: function get() {
      return {
        structure: function structure() {
          var folders = ['src/lib', 'web', 'tests'];

          folders.forEach(function (folder) {
            return (0, _mkdirp2.default)(folder);
          });
        },
        composerJson: function composerJson() {
          this.template('_composer.json', 'composer.json', {
            appname: this.appname,
            author: this.answers.author,
            email: this.answers.email
          });
        },
        phpunit: function phpunit() {
          this.template('phpunit.xml');
        },
        deployer: function deployer() {
          if (this.answers.doUseDeployer) {
            this.template('_deploy.php', 'deploy.php', { repo: this.answers.repo });
            this.template('_servers.yml', 'servers.yml', { deployer: this.answers.deployer });
          }
        },
        codeChecking: function codeChecking() {
          this.template('phpcs.xml');
        },
        ci: function ci() {
          if (this.answers.doUseCi) {
            this.template('_phpci.yml', 'phpci.yml', {
              email: this.answers.email,
              deployer: this.answers.deployer
            });
          }
        },
        other: function other() {
          this.template('gitignore', '.gitignore');
        }
      };
    }
  }, {
    key: 'install',
    get: function get() {
      return {
        composer: function composer() {
          this.spawnCommand('composer', ['install']) //TODO needs more gold(arguments)
          .on('error', function (err) {
            switch (err.code) {
              case 'ENOENT':
                console.error('Couldn\'t find composer binary, please specify your path.');
                break;
            }
          });
        },
        git: function git() {
          var _this3 = this;

          console.log('Initializing git repo ' + this.answers.repo);

          this.spawnCommand('git', ['init']).on('close', function (code) {
            if (!code) {
              _this3.spawnCommand('git', ['remote', 'add', 'origin', _this3.answers.repo]);
            }
          }).on('error', function (err) {
            switch (err.code) {
              case 'ENOENT':
                console.error('Couldn\'t find git binary, please specify your path.');
                break;
            }
          });
        }
      };
    }
  }]);

  return PhpGenerator;
}(_yeomanGenerator.Base);

exports.default = PhpGenerator;
module.exports = exports['default'];